# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Authors: Guido Amoruso <guidonte@fluendo.com>
#          Olivier Tilloy <olivier@fluendo.com>


from elisa.core import common

from elisa.core import media_uri

from elisa.core.utils.cancellable_defer import cancellable_deferred_iterator, \
        CancellableDeferred

from elisa.plugins.base.models.media import PlayableModel

from elisa.plugins.poblesec.actions import Action, LinkAction
from elisa.plugins.poblesec.actions import PlayAllAction, ShuffleAction
from elisa.plugins.poblesec.actions import AddToFavoritesAction, AddToPlaylistAction
from elisa.plugins.poblesec.actions import PlaySlideshowAction

from elisa.plugins.favorites.models import FavoritesItem
from elisa.plugins.database.models import Image, File
from twisted.internet import defer

import datetime
import os.path

from elisa.core.utils.i18n import install_translation

from elisa.plugins.poblesec.base.hierarchy import HierarchyController
from elisa.plugins.poblesec.base.list import GenericListViewMode
from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController
from elisa.plugins.poblesec.base.preview_list import \
    MenuItemPreviewListController, DoubleLineMenuItemPreviewListController
from elisa.plugins.poblesec.base.coverflow import \
    ImageWithReflectionCoverflowController
from elisa.plugins.poblesec.base.grid import GridItemGridController
from elisa.plugins.poblesec.link import Link

from elisa.plugins.favorites.models import FavoritesItem

from elisa.plugins.base.models.media import PlayableModel

from elisa.plugins.database.models import Video, File
from elisa.plugins.database import video_parser
from elisa.plugins.thetvdb import utils as tvdb_utils
from elisa.plugins.themoviedb import utils as themoviedb_utils

from twisted.internet import defer
from twisted.internet import task

import os, datetime
from random import shuffle

_ = install_translation('database')


class ArtistPlayAllAction(PlayAllAction):
    pass

class AlbumPlayAllAction(PlayAllAction):
    pass

class TrackPlayAllAction(PlayAllAction):

    def run(self):
        track = self.controller.track
        if track is not None:
            # Enqueue all the media after 'track' and play
            tracks = self.controller.tracks
            index = tracks.index(track)
            self.play(tracks[index:])
            return defer.succeed(None)
        else:
            dfr = super(TrackPlayAllAction, self).run()

        return dfr


class ArtistShuffleAction(ArtistPlayAllAction, ShuffleAction):
    pass

class AlbumShuffleAction(AlbumPlayAllAction, ShuffleAction):
    pass

class TrackShuffleAction(TrackPlayAllAction, ShuffleAction):
    pass


class AlbumAddToFavoritesAction(AddToFavoritesAction):

    def run(self):
        if self.toggled:
            if self.controller.artist:
                uri = getattr(self.controller.artist, 'uri', None)
                if uri:
                    dfr = common.application.store.find(FavoritesItem,
                                                        FavoritesItem.uri == unicode(uri),
                                                        FavoritesItem.type == u'artist')
                else:
                    dfr = common.application.store.find(FavoritesItem,
                                                        FavoritesItem.foreign_id == self.controller.artist.name,
                                                        FavoritesItem.foreign_class == u'Artist')
                dfr.addCallback(lambda rs: rs.one())
                dfr.addCallback(self.remove)
        else:
            favorite = FavoritesItem()
            favorite.added_time = datetime.datetime.now()

            # expecting an AlbumsController
            if self.controller.artist:
                uri = getattr(self.controller.artist, 'uri', None)
                if uri:
                    favorite.uri = unicode(uri)
                else:
                    favorite.foreign_id = self.controller.artist.name
                    favorite.foreign_class = u'Artist'
                favorite.type = u'artist'
                favorite.title = self.controller.artist.name
                self.store(favorite)


class TrackAddToFavoritesAction(AddToFavoritesAction):

    def run(self):
        if self.toggled:
            if self.controller.track:
                uri = getattr(self.controller.track, 'uri', None)
                if uri:
                    dfr = common.application.store.find(FavoritesItem,
                                                        FavoritesItem.uri == unicode(uri),
                                                        FavoritesItem.type == u'music_track')
                else:
                    dfr = common.application.store.find(FavoritesItem,
                                                        FavoritesItem.foreign_id == self.controller.track.file_path,
                                                        FavoritesItem.foreign_class == u'MusicTrack')
                dfr.addCallback(lambda rs: rs.one())
                dfr.addCallback(self.remove)
            elif self.controller.album:
                uri = getattr(self.controller.album, 'uri', None)
                if uri:
                    dfr = common.application.store.find(FavoritesItem,
                                                        FavoritesItem.uri == unicode(uri),
                                                        FavoritesItem.type == u'music_album')
                else:
                    dfr = common.application.store.find(FavoritesItem,
                                                        FavoritesItem.foreign_id == self.controller.album.name,
                                                        FavoritesItem.foreign_class == u'MusicAlbum')
                dfr.addCallback(lambda rs: rs.one())
                dfr.addCallback(self.remove)
        else:
            favorite = FavoritesItem()
            favorite.added_time = datetime.datetime.now()

            # expecting a TracksController
            if self.controller.track:
                uri = getattr(self.controller.track, 'uri', None)
                if uri:
                    favorite.uri = unicode(uri)
                else:
                    favorite.foreign_id = self.controller.track.file_path
                    favorite.foreign_class = u'MusicTrack'
                favorite.type = u'music_track'
                favorite.title = self.controller.track.title
                self.store(favorite)
            elif self.controller.album:
                uri = getattr(self.controller.album, 'uri', None)
                if uri:
                    favorite.uri = unicode(uri)
                else:
                    favorite.foreign_id = self.controller.album.name
                    favorite.foreign_class = u'MusicAlbum'
                favorite.type = u'music_album'
                favorite.title = self.controller.album.name
                self.store(favorite)

class PhotoAddToFavoritesAction(AddToFavoritesAction):

    def run(self):
        if self.toggled:
            if self.controller.album:
                dfr = common.application.store.find(FavoritesItem,
                                                    FavoritesItem.foreign_id == self.controller.album.name,
                                                    FavoritesItem.foreign_class == u'PhotoAlbum')
                dfr.addCallback(lambda rs: rs.one())
                dfr.addCallback(self.remove)
            elif self.controller.photo:
                dfr = common.application.store.find(FavoritesItem,
                                                    FavoritesItem.foreign_id == self.controller.photo.file_path,
                                                    FavoritesItem.foreign_class == u'Image')
                dfr.addCallback(lambda rs: rs.one())
                dfr.addCallback(self.remove)
        else:
            favorite = FavoritesItem()
            favorite.added_time = datetime.datetime.now()

            # expecting a TracksController
            if self.controller.album:
                favorite.foreign_id = self.controller.album.name
                favorite.foreign_class = u'PhotoAlbum'
                favorite.title = self.controller.album.name
                self.store(favorite)
            elif self.controller.photo:
                favorite.foreign_id = self.controller.photo.file_path
                favorite.foreign_class = u'Image'
                # FIXME: find a better title
                favorite.title = os.path.basename(self.controller.photo.file_path)
                self.store(favorite)


class ArtistAddToPlaylistAction(AddToPlaylistAction):

    def run(self):
        raise NotImplementedError()

class AlbumAddToPlaylistAction(AddToPlaylistAction):

    def run(self):
        raise NotImplementedError()

class TrackAddToPlaylistAction(AddToPlaylistAction):

    def run(self):
        raise NotImplementedError()

class ViewPhotoAlbumSlideshowAction(PlaySlideshowAction):

    def _order_photos(self, rs):
        # copied and pasted the ordering from database.PhotosController
        rs.order_by(Image.shot_time, Image.file_path)
        return rs.all()

    def _get_photos_from_album(self, album):
        dfr = common.application.store.find(Image, Image.album_name == album.name)
        dfr.addCallback(self._order_photos)
        return dfr

    def _load_images_from_database(self):
        items = [item for item in self.controller.model if not isinstance(item, Action)]
        images = []
        dfrs = []
        for item in items:
            dfr = self._get_photos_from_album(item)
            dfr.addCallback(lambda the_images: images.extend(the_images))
            dfrs.append(dfr)

        if len(dfrs) == 0:
            return defer.fail(None)

        def return_images(result):
            return images

        dfr_list = defer.DeferredList(dfrs)
        dfr_list.addCallback(return_images)
        return dfr_list

    def run(self):
        dfr = self._load_images_from_database()
        dfr.addCallback(self._play_slideshow)

class ViewPhotoDateSlideshowAction(PlaySlideshowAction):

    def order_photos(self, rs):
        # copied and pasted the ordering from database.PhotosController
        rs.order_by(Image.shot_time)
        return rs.all()

    def run(self):
        dfr = common.application.store.find(Image, Image.shot_time != None)
        dfr.addCallback(self.order_photos)
        dfr.addCallback(self._play_slideshow)
        return dfr

class RemoveAction(Action):

    def hide_stuff(self, dfr):
        def got_file(result):
            if result:
                result.hidden = True
            browser = self.controller.frontend.retrieve_controllers('/poblesec/browser')[0]
            browser.history.go_back()

            prev_controller = browser.history.current
            list_controller = prev_controller.list_controller
            del list_controller.model[list_controller.nodes.selected_item_index]

        dfr.addCallback(got_file)
        return dfr

class PhotoRemoveAction(RemoveAction):
    icon = 'elisa.plugins.poblesec.remove'

    def run(self):
        return self.hide_stuff(self.controller.photo.file)

class TrackRemoveAction(RemoveAction):
    icon = 'elisa.plugins.poblesec.remove'

    def run(self):
        return self.hide_stuff(self.controller.track.file)

class VideoPlayAllAction(Action):

    icon = 'elisa.plugins.poblesec.play_all'
    
    @cancellable_deferred_iterator
    def _build_playlist(self, videos):
        if hasattr(self.controller, 'db_model'):
            videos.append(self.controller.db_model)
            yield None
        elif hasattr(self.controller, 'file_path'):
            playable_model = PlayableModel()
            playable_model.uri = media_uri.MediaUri({'scheme' : 'file',
                                                     'path' : self.controller.file_path})
            videos.append(playable_model)
            yield None
        else:    
            for item in self.controller.model:
                if isinstance(item, Action):
                    continue
                videos.append(item)
                yield None
    
    def run(self):
        videos = []
        cancellable_iterator = self._build_playlist(videos)
        def canceller(dfr):
            cancellable_iterator.cancel()
        cancellable = CancellableDeferred(canceller)

        dfr = task.coiterate(iter(cancellable_iterator))
        dfr.addCallback(lambda result: self.order_videos(videos))
        dfr.addCallback(self.play)

        dfr.chainDeferred(cancellable)
        return cancellable
    
    def play(self, items):
    
        if not items:
            return items
    
        poblesec = self.controller.frontend.retrieve_controllers('/poblesec')[0]
        player = poblesec.video_player.player
        poblesec.show_video_player()

        player.play_model(items[0])
        for item in items[1:]:
            player.enqueue_to_playlist(item)
   
    def order_videos(self, videos):
        return videos

class VideoShuffleAction(VideoPlayAllAction):

    icon = 'elisa.plugins.poblesec.shuffle'
    
    def order_videos(self, videos):
        shuffle(videos)
        return videos
                
class VideoAddToFavoritesAction(AddToFavoritesAction):
    foreign_class = u'Video'

    def run(self):
        if self.toggled:
            if self.controller.name:
                dfr = common.application.store.find(FavoritesItem,
                                                    FavoritesItem.foreign_id == self.controller.file_path,
                                                    FavoritesItem.foreign_class == self.foreign_class)
                dfr.addCallback(lambda rs: rs.one())
                dfr.addCallback(self.remove)
        else:
            favorite = FavoritesItem()
            favorite.added_time = datetime.datetime.now()

            if self.controller.name:
                favorite.foreign_id = self.controller.file_path
                favorite.foreign_class = self.foreign_class
                favorite.title = self.controller.title
                self.store(favorite)

class MovieAddToFavoritesAction(VideoAddToFavoritesAction):
    foreign_class = u'Movie'

class TVEpisodeAddToFavoritesAction(VideoAddToFavoritesAction):
    foreign_class = u'TVEpisode'


class VideoRemoveAction(RemoveAction):
    icon = 'elisa.plugins.poblesec.remove'


class MovieViewSynopsisAction(LinkAction):
    icon = 'elisa.plugins.poblesec.synopsis'
    path = '/poblesec/database/movie/synopsis'


class TVEpisodeViewSynopsisAction(LinkAction):
    icon = 'elisa.plugins.poblesec.synopsis'
    path = '/poblesec/database/tvepisode/synopsis'

class VideoChangeCategoryAction(LinkAction):
    icon = 'elisa.plugins.poblesec.directory_settings_add_folder'
    path = '/poblesec/database/video/recategorize'

    def run(self):
        browser = self.controller.frontend.retrieve_controllers('/poblesec/browser')[0]
        browser.history.set_mark('recategorize')
        return super(VideoChangeCategoryAction, self).run()

class ToTVShowAction(Action):

    def run(self):
        file_path = self.controller.file_path

        store = common.application.store
        parser = video_parser.VideoParser(store)

        def got_search_results(tvdb_model):
            if tvdb_model and tvdb_model.series:
                controller_path = '/poblesec/database/recategorize/tvshow'
                args = dict(file_path=file_path, notifiers=self.controller.notifiers,
                            series=tvdb_model.series)
                label = _("TV Shows")
            else:
                controller_path = '/poblesec/database/recategorize/search/tvshow'
                args = dict(file_path=file_path, notifiers=self.controller.notifiers)
                label = _("Search")
            browser = self.controller.frontend.retrieve_controllers('/poblesec/browser')[0]
            history = browser.history
            return history.append_controller(controller_path, label, **args)

        tvshow_match = parser.try_tvshow_match(file_path)
        if tvshow_match:
            show_name, season_nb, episode_nb, remain = tvshow_match.groups()
        else:
            show_name = file_path
        cleaned_video_name = parser.clean_video_name(show_name)
        lookup_url = tvdb_utils.get_series_lookup_url(cleaned_video_name)
        model, dfr = common.application.resource_manager.get(lookup_url)
        dfr.addCallback(got_search_results)
        return dfr


class EndCategoryChangeAction(Action):

    def run(self):
        browser = self.controller.frontend.retrieve_controllers('/poblesec/browser')[0]
        return browser.history.goto_mark('recategorize')

class GoBackAction(Action):

    def __init__(self, controller, new_controller_header_text):
        super(GoBackAction, self).__init__(controller)
        self.new_controller_header_text = new_controller_header_text

    def run(self):
        browser = self.controller.frontend.retrieve_controllers('/poblesec/browser')[0]
        history = browser.history

        def controller_removed(controller):
            history.current.title_text = self.new_controller_header_text

        dfr = history.go_back()
        dfr.addCallback(controller_removed)
        return dfr

class ConfirmCategoryChangeAction(EndCategoryChangeAction):
    confirm = None

    def run(self):
        dfr = self.confirm(self)

        def run_parent(result):
            return super(ConfirmCategoryChangeAction, self).run()

        def go_back(result):
            browser = self.controller.frontend.retrieve_controllers('/poblesec/browser')[0]
            # go back from new category selection list to the list of videos
            return browser.history.go_back()

        dfr.addCallback(run_parent)
        dfr.addCallback(go_back)
        return dfr

class CancelCategoryChangeAction(EndCategoryChangeAction):
    pass

class ManualRecategorizeAction(Action):
    icon = 'elisa.plugins.poblesec.manual_write'
    controller_path = '/poblesec/database/recategorize/manual/tvshow'
    crumb_title = _("Enter TV Show name")

    def run(self):
        frontend = self.controller.frontend
        controllers = frontend.retrieve_controllers('/poblesec/browser')
        history = controllers[0].history

        dfr = history.append_controller(self.controller_path,
                                        self.crumb_title,
                                        file_path=self.controller.file_path,
                                        prefilled_text=self.controller.search_term,
                                        notifiers=self.controller.notifiers)
        return dfr








###############################################################################
#
# NEW GENERATION CONTEXTUAL ACTIONS
#
###############################################################################

from elisa.core.media_uri import MediaUri

from elisa.plugins.poblesec.actions import ContextualAction, \
                                           OpenControllerAction

from elisa.plugins.database.models import Artist, MusicAlbum, \
                                          PICTURES_SECTION, TVEpisode

from elisa.plugins.base.models.media import PlaylistModel
from elisa.plugins.base.models.image import ImageModel


class EnqueueAndPlayMusicMixin(object):

    def enqueue_and_play(self, tracks, index=0):
        """
        Generic method to call for enqueuing and playing music.

        We pass a list of tracks that we want to enqueue and, eventually, the
        index of a track inside this list to be played first.
        """
        playlist = PlaylistModel()
        playlist.extend(tracks)
        poblesec = self.controller.frontend.retrieve_controllers('/poblesec')[0]
        player = poblesec.music_player.player
        player.set_playlist(playlist)
        poblesec.show_music_player()
        player.play_at_index(index)


class EnqueueAndPlayVideosMixin(object):

    def enqueue_and_play(self, videos, index=0):
        """
        Generic method to call for enqueuing and playing videos.

        We pass a list of videos that we want to enqueue and, eventually, the
        index of a video inside this list to be played first.
        """
        playlist = PlaylistModel()
        playlist.extend(videos)
        poblesec = self.controller.frontend.retrieve_controllers('/poblesec')[0]
        player = poblesec.video_player.player
        player.set_playlist(playlist)
        poblesec.show_video_player()
        player.play_at_index(index)


class EnqueueAndPlayPicturesMixin(object):

    def _iterate(self, playlist, pictures):
        for picture in pictures:
            if isinstance(picture, ImageModel):
                playlist.append(picture)
            else:
                # Instantiate an ImageModel from a database Image model
                image = ImageModel()
                if hasattr(picture, "thumbnail") and picture.thumbnail:
                    thumbnail_uri = MediaUri('file://' + picture.thumbnail)
                    image.references.append(thumbnail_uri)
                image.references.append(MediaUri('file://' + picture.file_path))
                image.title = os.path.basename(picture.file_path)
                if picture.orientation is not None:
                    image.orientation = picture.orientation
                playlist.append(image)

            yield None

    def build_playlist(self, pictures):
        playlist = []
        dfr = task.coiterate(self._iterate(playlist, pictures))
        dfr.addCallback(lambda result: playlist)
        return dfr

    def enqueue_and_play(self, pictures, index=0):
        poblesec = self.controller.frontend.retrieve_controllers('/poblesec')[0]
        player = poblesec.slideshow_player.player
        player.set_playlist(pictures)
        poblesec.show_slideshow_player()
        player.start_slideshow(index)


class ShuffleContextualAction(ContextualAction, EnqueueAndPlayMusicMixin):

    name = 'shuffle'

    def execute(self, item):
        def got_tracks(tracks):
            shuffle(tracks)
            return self.enqueue_and_play(tracks)

        dfr = item.get_tracks()
        dfr.addCallback(got_tracks)
        return dfr


class InformationAction(ContextualAction):
    # Detailed information about an item (generic action, should be subclassed)

    name = 'info'
    label = _('Info')


class MoreOptionsAction(OpenControllerAction):
    # More options associated to an item (generic action, should be subclassed)

    name = 'more'
    label = _('More')
    path = ''

    def __init__(self, controller):
        OpenControllerAction.__init__(self, controller, self.path)


class PlayArtistAction(ContextualAction, EnqueueAndPlayMusicMixin):
    # Enqueue all tracks of all albums of the artist and start playing the
    # first one.
    # This is the default action for artists.

    def execute(self, item):
        # FIXME: using get_tracks may not be a good idea, albums will not be
        # ordered by name and tracks by number.
        dfr = item.get_tracks()
        dfr.addCallback(self.enqueue_and_play)
        return dfr


class ArtistTracksShuffleAction(ShuffleContextualAction):
    label = _('Shuffle and Play All Tracks')


class ListArtistAlbumsAction(OpenControllerAction):
    # List all the albums of a given artist

    name = 'list'
    label = _('List')

    def execute(self, item):
        return self.open_controller(self.path, item.name, artist=item)


class ArtistInformationAction(InformationAction):
    # Detailed information on an album

    # TODO: implement me
    pass


class ArtistMoreOptionsAction(MoreOptionsAction):
    # More options associated to an artist
    path = '/poblesec/database/artist/more'

    def execute(self, item):
        return self.open_controller(self.path, item.name, artist=item)


class PlayAlbumAction(ContextualAction, EnqueueAndPlayMusicMixin):
    # Enqueue all tracks in the album and start playing the first one.
    # This is the default action for music albums.

    def execute(self, item):
        dfr = item.get_tracks()
        dfr.addCallback(self.enqueue_and_play)
        return dfr


class ListAlbumTracksAction(OpenControllerAction):
    # List all tracks in a given album
    name = 'list'
    label = _('Tracks List')

    def __init__(self, controller):
        path = '/poblesec/database/album/list'
        super(ListAlbumTracksAction, self).__init__(controller, path)

    def execute(self, item):
        return self.open_controller(self.path, item.name, album=item)


class AlbumInformationAction(InformationAction):
    # Detailed information on an album

    # TODO: implement me
    pass


class AlbumMoreOptionsAction(MoreOptionsAction):
    # More options associated to an album
    path = '/poblesec/database/album/more'

    def execute(self, item):
        return self.open_controller(self.path, item.name, album=item)


class AlbumTracksShuffleAction(ShuffleContextualAction):
    label = _('Shuffle and Play Tracks')


class PlayTrackAction(ContextualAction, EnqueueAndPlayMusicMixin):
    """
    Enqueue and start playing a single track.
    This is the default action for the Generic Track screen.
    """

    def execute(self, item):
        self.enqueue_and_play([item])
        return defer.succeed(None)

class PlaySeveralTracksAction(ContextualAction, EnqueueAndPlayMusicMixin):
    """
    Enqueue the entire controller's list of tracks and start playing a single track.
    This is the default action for the Album Tracks screen, for instance.
    """

    def execute(self, item):
        index = self.controller.model.index(item)
        self.enqueue_and_play(self.controller.model, index)
        return defer.succeed(None)

class PlayVideoAction(ContextualAction, EnqueueAndPlayVideosMixin):
    """
    Enqueue and start playing a single video.
    This is the default action for videos.
    """

    def execute(self, item):
        self.enqueue_and_play([item])
        return defer.succeed(None)

class PlaySeveralVideosAction(ContextualAction, EnqueueAndPlayVideosMixin):
    """
    Enqueue the entire controller's list of videos and start playing a single video.
    This is the default action for the TV Series, for instance.
    """

    def execute(self, item):
        index = self.controller.model.index(item)
        self.enqueue_and_play(self.controller.model, index)
        return defer.succeed(None)

class OpenMovieSynopsisAction(OpenControllerAction):

    """
    Open the synopsis of a movie.
    """

    name = 'info'

    def __init__(self, controller):
        path = '/poblesec/database/movie/synopsis'
        super(OpenMovieSynopsisAction, self).__init__(controller, path)

    def execute(self, item):
        return self.open_controller(self.path, _('Synopsis'), movie=item,
                                    movies=self.controller.model)


class VideoMoreOptionsAction(MoreOptionsAction):
    # More options associated to a video

    # TODO: implement me
    pass


class ListSeasonEpisodesAction(OpenControllerAction):
    # List all the episodes in a TV show season.

    name = 'list'
    label = _('Episode List')

    def __init__(self, controller):
        path = '/poblesec/database/tvseason/list'
        super(ListSeasonEpisodesAction, self).__init__(controller, path)

    def _get_title(self, item):
        def got_show(show):
            return "%s Season %d" % (show.name, item.number)

        dfr = item.tvshow
        dfr.addCallback(got_show)
        return dfr

    def execute(self, item):
        def open_controller(title):
            return self.open_controller(self.path, title, season=item)

        dfr = self._get_title(item)
        dfr.addCallback(open_controller)
        return dfr


class TVSeasonMoreOptionsAction(MoreOptionsAction):
    # More options associated to a TV show season.
    path = '/poblesec/database/tvseason/more'

    def _get_title(self, item):
        def got_show(show):
            return "%s Season %d" % (show.name, item.number)

        dfr = item.tvshow
        dfr.addCallback(got_show)
        return dfr

    def execute(self, item):
        def open_controller(title):
            return self.open_controller(self.path, title, season=item)

        dfr = self._get_title(item)
        dfr.addCallback(open_controller)
        return dfr


class PlayTVSeasonAction(ContextualAction, EnqueueAndPlayVideosMixin):
    # Enqueue all episodes in the season and start playing the first one.

    label = _('Play Entire Season')

    def execute(self, item):
        dfr = item.episodes.order_by(TVEpisode.number)
        dfr.addCallback(lambda result_set: result_set.all())
        dfr.addCallback(self.enqueue_and_play)
        return dfr


class OpenTVEpisodeSynopsisAction(OpenControllerAction):

    """
    Open the synopsis of a TV episode.
    """

    name = 'info'

    def __init__(self, controller):
        path = '/poblesec/database/tvepisode/synopsis'
        super(OpenTVEpisodeSynopsisAction, self).__init__(controller, path)

    def execute(self, item):
        return self.open_controller(self.path, _('Synopsis'), episode=item,
                                    episodes=self.controller.model)


class PlayPictureAction(ContextualAction, EnqueueAndPlayPicturesMixin):
    # Enqueue all images and start the slideshow at the selected one.
    # This is the default action for photo controllers.

    def execute(self, item):
        index = self.controller.model.index(item)
        dfr = self.build_playlist(self.controller.model)
        dfr.addCallback(self.enqueue_and_play, index)
        return dfr


class PictureMoreOptionsAction(MoreOptionsAction):
    # More options associated to a picture

    # TODO: implement me
    pass


class PlayPictureAlbumAction(ContextualAction, EnqueueAndPlayPicturesMixin):
    # Enqueue all images in the album and start the slideshow at the first one.
    # This is the default action for photo album controllers.

    def execute(self, item):
        def order(result_set):
            result_set.order_by(Image.shot_time, Image.file_path)
            return result_set.all()

        dfr = item.photos.find()
        dfr.addCallback(order)
        dfr.addCallback(self.build_playlist)
        dfr.addCallback(self.enqueue_and_play)
        return dfr


class ListAlbumPicturesAction(OpenControllerAction):
    # List all the pictures in a given album

    name = 'list'
    label = _('List')

    def __init__(self, controller):
        path = '/poblesec/database/photo/list'
        super(ListAlbumPicturesAction, self).__init__(controller, path)

    def execute(self, item):
        return self.open_controller(self.path, item.name, album=item)


class PictureAlbumMoreOptionsAction(MoreOptionsAction):
    # More options associated to a picture album

    # TODO: implement me
    pass


class PlayPictureTimeAction(ContextualAction, EnqueueAndPlayPicturesMixin):
    # Enqueue all images in a given time frame (year/month) and start the
    # slideshow at the first one.
    # This is the default action for the photo times controller.

    def execute(self, item):
        store = common.application.store
        try:
            last = datetime.datetime(item.year, item.month + 1, 1)
        except ValueError:
            # We reached the end of the year, skip to January of the following
            # year.
            last = datetime.datetime(item.year + 1, 1, 1)

        def order(result_set):
            result_set.order_by(Image.shot_time, Image.file_path)
            return result_set.all()

        dfr = store.find(Image, Image.section == PICTURES_SECTION,
                         Image.shot_time >= item,
                         Image.shot_time < last,
                         Image.file_path == File.path, File.hidden == False)
        dfr.addCallback(order)
        dfr.addCallback(self.build_playlist)
        dfr.addCallback(self.enqueue_and_play)
        return dfr


class ListPictureTimeAction(OpenControllerAction):
    # List all the pictures in a given album

    name = 'list'
    label = _('List')

    def __init__(self, controller):
        path = '/poblesec/database/photo/by_month'
        super(ListPictureTimeAction, self).__init__(controller, path)

    def execute(self, item):
        try:
            title = item.strftime(_('%B, %Y'))
        except TypeError:
            # strftime in python <=2.5 requires a str
            title = item.strftime(_('%B, %Y').encode('utf-8', 'replace'))

        return self.open_controller(self.path, title,
                                    year=item.year, month=item.month)
