# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Florian Boucault <florian@fluendo.com>


from elisa.core.input_event import EventValue, EventSource, UnicodeInputEvent
from elisa.plugins.pigment.graph.text import Text
from elisa.plugins.pigment.widgets.button import AbstractButton
import gobject


class TextEntry(AbstractButton):
    """
    A text entry widget, displaying a single line input.

    Emit these signals:
      - content-changed: when the text in the entry has changed

    @ivar content: the content of the entry
    @type content: unicode
    @ivar label:   DOCME
    @type label:   DOCME  
    """

    __gsignals__ = {'content-changed': (gobject.SIGNAL_RUN_LAST, 
                                        gobject.TYPE_BOOLEAN,
                                        ())
                   }

    def __init__(self):
        super(TextEntry, self).__init__()
        self._create_widgets()
        self.update_style_properties(self.style.get_items())

    def _create_widgets(self):
        self.label = Text()
        self.label.visible = True
        self.add(self.label)
        # FIXME: not pretty
        self._connect_mouse_events(self.label)

    def content__get(self):
        return unicode(self.label.label)

    def content__set(self, value):
        value = value or ''
        if self.label.label != value:
            self.label.label = value
            self.emit('content-changed')

    content = property(content__get, content__set)

    def handle_input(self, manager, event):
        if event.value == EventValue.KEY_SPACE:
            self.content += ' '
            return True
        elif event.value == EventValue.KEY_MENU:
            if event.source == EventSource.KEYBOARD and len(self.content) > 0:
                    self.content = self.content[0:-1]
                    return True
            else:
                return False
        elif isinstance(event, UnicodeInputEvent):
            self.content += event.value
            return True
        elif event.value != EventValue.KEY_OK and \
                 str(event.value).startswith(u'KEY_'):
            letter = str(event.value)[4:]
            if letter.isalnum():
                self.content += letter
                return True

        return super(TextEntry, self).handle_input(manager, event)

    @classmethod
    def _demo_widget(cls, *args, **kwargs):
        widget = cls()
        widget.size = (100.0, 10.0)
        widget.content = "Type in something..."
        widget.visible = True
        return widget


if __name__ == '__main__':
    from elisa.plugins.pigment.widgets.input import TextEntry
    TextEntry.demo()
    try:
        __IPYTHON__
    except NameError:
        import pgm
        pgm.main()
