# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.

from elisa.plugins.pigment.graph.text import Text
from elisa.plugins.pigment.graph.image import Image
from elisa.plugins.pigment.widgets.widget import Widget
from elisa.plugins.pigment.widgets.button import AbstractButton
from elisa.plugins.pigment.widgets.box import HBox

from elisa.plugins.poblesec.widgets.button import RelookSmoothPanel
from elisa.plugins.poblesec.widgets.artwork_box import ArtworkBox

from pgm.timing import implicit


class MenuItemWidget(AbstractButton):

    def __init__(self, with_artwork_box=True):
        super(MenuItemWidget, self).__init__()
        self._with_artwork_box = with_artwork_box
        self._create_widgets()
        self.update_style_properties(self.style.get_items())

    def _create_labels(self):
        labels = Widget()
        labels.visible = True

        self.label = Text()
        self.label.visible = True
        labels.add(self.label)

        return labels

    def _create_artwork_box(self):
        artwork_box = ArtworkBox()
        artwork_box.visible = True
        return artwork_box

    def _create_widgets(self):
        self.foreground = HBox()
        self.foreground.visible = True
        self.add(self.foreground, forward_signals=False)

        self.artwork_box = self._create_artwork_box()
        if self._with_artwork_box:
            self.foreground.pack_start(self.artwork_box, expand=False)

        self.labels = self._create_labels()
        self.foreground.pack_end(self.labels, expand=True)

        # background drawable taking the full size of the widget; used to
        # catch mouse events
        self.background = Image()
        self.add(self.background, forward_signals=False)
        self.background.visible = True
        self._connect_mouse_events(self.background)

        self.panel = RelookSmoothPanel()
        self.add(self.panel, forward_signals=False)
        self.panel.visible = True

    def _set_with_artwork_box(self, value):
        if self._with_artwork_box != value:
            self._with_artwork_box = value
            if value:
                self.foreground.pack_start(self.artwork_box, expand=False)
            else:
                self.foreground.remove(self.artwork_box)

    def _get_with_artwork_box(self):
        return self._with_artwork_box

    with_artwork_box = property(fget=_get_with_artwork_box, \
                                fset=_set_with_artwork_box)

    def do_state_changed(self, previous_state):
        super(MenuItemWidget, self).do_state_changed(previous_state)
        self.panel.state = self.state


class DoubleLineMenuItemWidget(MenuItemWidget):

    def _create_labels(self):
        labels = super(DoubleLineMenuItemWidget, self)._create_labels()

        sublabel = Text()
        sublabel.visible = True
        labels.add(sublabel)
        self.sublabel = sublabel

        return labels

class ThreeLineMenuItemWidget(DoubleLineMenuItemWidget):

    # A menu item widget with an icon and three lines of text.

    def _create_labels(self):
        labels = super(ThreeLineMenuItemWidget, self)._create_labels()

        sublabel2 = Text()
        labels.add(sublabel2)
        sublabel2.visible = True
        self.sublabel2 = sublabel2

        return labels
